const { PrismaClient } = require('@prisma/client')
const express = require("express");
const bunyan = require('bunyan');
const cors = require("cors");
const axios = require('axios');
const app = express();

const prisma = new PrismaClient()

app.use(cors(corsOptions));
app.use(express.json());
app.use(express.urlencoded({ extended: true }));

var corsOptions = {
  origin: "http://localhost:8081"
};

const log = bunyan.createLogger({ name: "AIR_UI_Backend: " });
const predictionURL = 'https://air-fastapi.azurewebsites.net/predict'

function convertDataSizeToKb(textInput, data) {
  const dataInKb = (new TextEncoder().encode(JSON.stringify(data)).length) / 1024
  console.log(textInput, dataInKb)
}

function dataPreparationForML(age, gender /*, assistiveAids*/) {
  const Gender = gender == 'Mand' ? 1 : 0
  const BirthYear = (new Date().getFullYear() - age)
  const LoanPeriod = 180
  const Ats = "222718,093307,181210,120606,222718,093307,181210"
  //Use real assistive aids and loan period when possible

  return {
    "Gender": Gender,
    "BirthYear": BirthYear,
    "LoanPeriod": LoanPeriod,
    "Ats": Ats
  }
}

function getPrediction(inputForMLPrediction) {
  predictionResult = axios.post(predictionURL, inputForMLPrediction).then(result => {
    return result.data;
  }).catch(error => {
    log.error("getPrediction() threw an exception: ", error)
  })
  return predictionResult
}

function getCitizenInformation(citizenSSN) {
  const citizen = prisma.citizen.findUnique({
    where: {
      ssn: citizenSSN
    },
    select: {
      first_name: true,
      last_name: true,
      age: true,
      ssn: true,
      gender: true,
      Training_Plans: {
        select: {
          agreement_date: true,
          name: true,
          type: true,
          start_date: true,
          end_date: true,
          status: true,
        },
      },
      Home_Care: {
        select: {
          date: true,
          type: true,
          name: true,
          time_slot: true,
        },
      },
      Registered_Falls: {
        select: {
          date_fall: true,
          organisation_name: true,
          observation_name: true,
          number_of_affected_citizens: true,
        },
      },
    },
  })
  return citizen
}

app.post("/home/:ssn", async (req, res) => {
  const citizenData = await getCitizenInformation(req.body.citizenSSN);
  if (!citizenData) {
    res.send(citizenData)
  } else {
    const inputDataForMLPrediction = dataPreparationForML(citizenData.age, citizenData.gender);
    const predictionFromML = await getPrediction(inputDataForMLPrediction);
    convertDataSizeToKb("Data size from ML and Database in Kb: ", { citizenData, predictionFromML })
    res.json({ citizenData, predictionFromML })
  }
});

const PORT = process.env.PORT || 8080;
app.listen(PORT, () => {
  log.info(`Server is running on port ${PORT}.`);
});